import { Component, Input, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { EventsResultsWrapper } from 'src/app/common/models/EventsResultsWrapper';
import { Event } from '../../../common/models/Event';
import { GetQuery } from '../../../common/models/get-query';
import { EventsService } from '../../../common/services/events.service';

@Component({
  selector: 'app-events-list',
  templateUrl: './events-list.component.html',
  styleUrls: ['./events-list.component.scss']
})
export class EventsListComponent implements OnInit {
  /**
   * Searchterm
   */
  searchTerm = '';
  /**
   * Events to be displayed
   */
  events: Event[] = [];
  /**
   * Rsvps of current user
   */
  rsvps = {};

  constructor(
    private eventsService: EventsService,
    private router: Router
  ) { }

  /**
   * Fetches all events
   */
  ngOnInit() {
    this.fetchDate$().subscribe((events) => {
      this.events = events.results;
      this.rsvps = events.rsvp;
    });
  }

  /**
   * Fetch event with searchterm
   */
  fetchDate$(): Observable<EventsResultsWrapper> {
    return this.eventsService.getEvents(
      new GetQuery()
        .limit(0)
        .page(0)
        .like('description', this.searchTerm)
        .or()
        .like('title', this.searchTerm)
    ).pipe(
      map((wrapper) => wrapper)
    );
  }

  /**
   * Handles input change on search input.
   * Refetches events after some time
   * @param term
   */
  handleSearchChange(term: string) {
    this.searchTerm = term;
    this.fetchDate$().subscribe((eventsWrapper) => {
      this.events = eventsWrapper.results;
      this.rsvps = eventsWrapper.rsvp;
    });
  }

  /**
   * Navigates to event details page
   * @param uuid
   */
  navigateToEventDetailsPage(uuid: string) {
    this.router.navigate([`/events/${uuid}`]);
  }

  /**
   * Returns the events rsvp
   * @param event
   */
  eventRsvp(event: Event): string {
    if (!this.events || this.events.length <= 0 || this.rsvps === null || this.rsvps === undefined) {
      return null;
    }
    const rsvpEventMatchingEvent = this.rsvps[event.id];
    if (rsvpEventMatchingEvent) {
      return rsvpEventMatchingEvent.status;
    }

    return null;
  }
}
